package com.cultural.heritage.service.good.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.CartRecordMapper;
import com.cultural.heritage.mapper.GoodMapper;
import com.cultural.heritage.model.dto.cart.CartRecordAddRequest;
import com.cultural.heritage.model.entity.CartRecord;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.vo.cart.CartRecordVO;
import com.cultural.heritage.model.vo.good.GoodVO;
import com.cultural.heritage.service.good.CartRecordService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.user.UserService;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;

@Service
public class CartRecordServiceImpl extends ServiceImpl<CartRecordMapper, CartRecord> implements CartRecordService {


    @Resource
    private GoodMapper goodMapper;


    @Resource
    private GoodService goodService;


    @Resource
    private UserService userService;

    /**
     * 处理用户购物车已存在该商品
     */
    @Override
    public boolean dealAlreadyExistGood(CartRecordAddRequest cartRecordAddRequest, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Long goodId = cartRecordAddRequest.getGoodId();
        QueryWrapper<CartRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        queryWrapper.eq("goodId", goodId);
        CartRecord cartRecord = this.baseMapper.selectOne(queryWrapper);
        if (cartRecord != null) {
            cartRecord.setQuantity(cartRecord.getQuantity() + cartRecordAddRequest.getQuantity());
            cartRecord.setSubtotal(cartRecord.getSubtotal().add(cartRecordAddRequest.getSubtotal()));
            boolean result = this.updateById(cartRecord);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        } else {
            CartRecord cartGood = new CartRecord();
            BeanUtils.copyProperties(cartRecordAddRequest, cartGood);
            cartGood.setUserId(userId);
            // 校验
            this.validCart(cartGood, false);
            boolean result = this.save(cartGood);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        }
        return true;
    }





    /**
     * 封装用户购物车中的商品信息
     */
    @Override
    public List<CartRecordVO> transformToCartRecordVOList(List<CartRecord> list) {
        return list.stream().map(cartRecord -> {
            // 封装购物车记录VO
            CartRecordVO cartRecordVO = new CartRecordVO();
            BeanUtils.copyProperties(cartRecord, cartRecordVO);

            // 封装商品类GoodVO
            Long goodId = cartRecord.getGoodId();
            QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("id", goodId);
            Good good = goodMapper.selectOne(queryWrapper);
            GoodVO goodVO = new GoodVO();
            BeanUtils.copyProperties(good, goodVO);
            cartRecordVO.setGoodVO(goodVO);
            return cartRecordVO;
        }).toList();
    }



    /**
     * 校验购物车中的商品信息
     */
    @Override
    public void validCart(CartRecord cartRecord, boolean update) {
        Long userId = cartRecord.getUserId();
        Long goodId = cartRecord.getGoodId();
        Integer quantity = cartRecord.getQuantity();
        BigDecimal subtotal = cartRecord.getSubtotal();
        Integer isGoodType = cartRecord.getIsGoodType();
        if (update) {
            if (userId == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR);
            }
        }
        Good good = goodService.getById(goodId);
        if (good == null || good.getIsShelves() == 0) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "不存在该商品或该商品已下架");
        }
        if (ObjectUtils.isEmpty(quantity) || quantity < 1) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请传递正确的商品数量");
        }
        if (ObjectUtils.isEmpty(subtotal) || subtotal.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "价格错误");
        }
        if (ObjectUtils.isEmpty(isGoodType) || isGoodType != 1 && isGoodType != 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请传递正确的商品类别");
        }

    }
}
