package com.cultural.heritage.service.good.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.mapper.CouponMapper;
import com.cultural.heritage.model.dto.coupon.CouponQueryRequest;
import com.cultural.heritage.model.entity.Coupon;
import com.cultural.heritage.service.good.CouponService;
import com.cultural.heritage.utils.SqlUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;

@Service
public class CouponServiceImpl extends ServiceImpl<CouponMapper, Coupon> implements CouponService {


    @Override
    public QueryWrapper<Coupon> getQueryWrapper(CouponQueryRequest couponQueryRequest) {
        if (couponQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = couponQueryRequest.getId();
        String name = couponQueryRequest.getName();
        Date startTime = couponQueryRequest.getStartTime();
        Date endTime = couponQueryRequest.getEndTime();
        String sortField = couponQueryRequest.getSortField();
        String sortOrder = couponQueryRequest.getSortOrder();
        QueryWrapper<Coupon> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(id), "id", id);
        queryWrapper.like(StringUtils.isNotBlank(name), "name", name);
        queryWrapper.ge(ObjectUtils.isNotEmpty(startTime), "startTime", startTime);
        queryWrapper.le(ObjectUtils.isNotEmpty(endTime), "endTime", endTime);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }

    @Override
    public Integer verifyIsReachTheUpperLimit(Long userId, Long couponId) {
        return this.baseMapper.verifyIsReachTheUpperLimit(userId, couponId);
    }


    /**
     * 校验
     */
    @Override
    public void validCoupon(Coupon coupon, boolean update) {
        Long id = coupon.getId();
        String name = coupon.getName();
        BigDecimal conditionAmount = coupon.getConditionAmount();
        Integer requirePoints = coupon.getRequirePoints();
        Integer totalNum = coupon.getTotalNum();
        Integer residueNum = coupon.getResidueNum();
        Integer limitNum = coupon.getLimitNum();
        String useScope = coupon.getUseScope();
        String startTime = coupon.getStartTime();
        String endTime = coupon.getEndTime();
        String intro = coupon.getIntro();
        String image = coupon.getImage();
        String description = coupon.getDescription();
        if (update) {
            if (id == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数id错误");
            }
        }
        if (ObjectUtils.isEmpty(conditionAmount) || conditionAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "满减价格错误");
        }
        if (ObjectUtils.isEmpty(requirePoints) || requirePoints <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "积分参数错误");
        }
        if (ObjectUtils.isEmpty(totalNum) || ObjectUtils.isEmpty(residueNum) || ObjectUtils.isEmpty(limitNum)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "优惠券数量参数为null");
        }
        if (totalNum <= 0 || residueNum < 0 || residueNum > totalNum || limitNum <= 0 || limitNum > totalNum) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "优惠券数量参数错误");
        }
        if (StringUtils.isAnyBlank(name, useScope, startTime, endTime, intro, image, description)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在参数为空");
        }
    }
}
